@@ -1,5 +1,6 @@
 import logging
 import re
+from time import sleep
 from typing import Callable, Any, Tuple, List, Optional
 
 from getdeck.configuration import ClientConfiguration
@@ -7,63 +8,85 @@ from getdeck.configuration import ClientConfiguration
 logger = logging.getLogger("deck")
 
 
+def _call_and_log(config, api, verb, obj, namespace, **kwargs):
+    response = k8s_call_api(config, api, verb, obj, namespace, **kwargs)
+    if api:
+        logger.debug(
+            f"Kubernetes: {k8s_describe_object(obj)} created with uid={response.metadata.uid}"
+        )
+
+
+def _delete_and_create(config, api, obj, namespace, **kwargs):
+    logger.debug(
+        f"Kubernetes: replacing {k8s_describe_object(obj)} "
+        f"failed. Attempting deletion and recreation."
+    )
+    k8s_call_api(config, api, "delete", obj, namespace, **kwargs)
+    if api:
+        logger.debug(f"Kubernetes: {k8s_describe_object(obj)} deleted")
+    _call_and_log(config, api, "create", obj, namespace, **kwargs)
+
+
 def k8s_create_or_patch(
     config: ClientConfiguration, obj, namespace: str, **kwargs
 ) -> None:
     from kubernetes.client.rest import ApiException
 
     api = k8s_select_api(config, obj)
-    try:
-        res = k8s_call_api(config, api, "create", obj, namespace, **kwargs)
-        if api:
-            logger.debug(
-                f"Kubernetes: {k8s_describe_object(obj)} created with uid={res.metadata.uid}"
-            )
-    except ApiException as e:
-        if e.reason != "Conflict":
-            raise
+    # some objects require a k8s extension which might not be ready at the first try
+    # so, this is a retry pattern
+    for i in range(0, config.K8S_OBJECT_RETRY):
         try:
-            res = k8s_call_api(config, api, "patch", obj, namespace, **kwargs)
-            if api:
-                logger.debug(
-                    f"Kubernetes: {k8s_describe_object(obj)} patched with uid={res.metadata.uid}"
-                )
+            _call_and_log(config, api, "create", obj, namespace, **kwargs)
+            break
         except ApiException as e:
-            if e.reason != "Unprocessable Entity":
-                logger.error(
-                    f"Error installing object {obj['metadata']['name']}: {e.reason}"
-                )
-                raise
-            try:
-                # try again
-                logger.debug(
-                    f"Kubernetes: replacing {k8s_describe_object(obj)} "
-                    f"failed. Attempting deletion and recreation."
-                )
-                res = k8s_call_api(config, api, "delete", obj, namespace, **kwargs)
-                if api:
-                    logger.debug(f"Kubernetes: {k8s_describe_object(obj)} deleted")
-                res = k8s_call_api(config, api, "create", obj, namespace, **kwargs)
-                if api:
+            if e.reason in ["Not Found", "Internal Server Error"]:
+                logger.debug(e)
+                try:
+                    # try to create this object as non-namespaced object
+                    k8s_call_api(config, api, "create", obj, None, **kwargs)
+                    break
+                except ApiException as ie:
+                    logger.debug(ie)
+                if i < config.K8S_OBJECT_RETRY - 1:
                     logger.debug(
-                        f"Kubernetes: {k8s_describe_object(obj)} created with uid={res.metadata.uid}"
+                        f"This is attempt {i} (of {int(config.K8S_OBJECT_RETRY) - 1}), trying again"
                     )
-            except Exception as ex:
-                if api:
-                    logger.error(
-                        f"Kubernetes: failure updating {k8s_describe_object(obj)}: {ex}"
+                    sleep(config.K8S_OBJECT_RETRY_TIMEOUT)
+                    continue
+                else:
+                    raise RuntimeError(e)
+            elif e.reason == "Conflict":
+                try:
+                    _call_and_log(config, api, "patch", obj, namespace, **kwargs)
+                    break
+                except ApiException as e:
+                    if e.reason != "Unprocessable Entity":
+                        logger.error(
+                            f"Error installing object {obj['metadata']['name']}: {e.reason}"
+                        )
+                        raise RuntimeError(e)
+                    try:
+                        # try to delete this object and recreate it
+                        _delete_and_create(config, api, obj, namespace)
+                        break
+                    except Exception as ex:
+                        logger.error(
+                            f"Kubernetes: failure updating {k8s_describe_object(obj)}: {ex}"
+                        )
+                        raise RuntimeError(ex)
+                except ValueError as e:
+                    logger.debug(
+                        f"Error installing object {obj['metadata']['name']}: {e}. "
+                        f"This is probably caused by a skew in the Kubernetes version."
                     )
-                raise RuntimeError(ex)
+            else:
+                raise RuntimeError(e)
         except ValueError as e:
-            logger.warning(
+            logger.debug(
                 f"Error installing object {obj['metadata']['name']}: {e}. "
                 f"This is probably caused by a skew in the Kubernetes version."
             )
-    except ValueError as e:
-        logger.warning(
-            f"Error installing object {obj['metadata']['name']}: {e}. "
-            f"This is probably caused by a skew in the Kubernetes version."
-        )
 
 
 def k8s_delete_object(config: ClientConfiguration, obj, namespace, **kwargs) -> bool:
@@ -103,13 +126,13 @@ def k8s_select_api(config: ClientConfiguration, obj) -> Optional[Callable]:
 
 
 def k8s_call_api(
-    config: ClientConfiguration, api, action, obj, namespace: str, **args
+    config: ClientConfiguration, api, action, obj, namespace: Optional[str], **args
 ) -> Any:
     kind = convert_camel_2_snake(obj["kind"])
     try:
         namespace = obj["metadata"]["namespace"]
     except KeyError:
-        # take the namespace passed as installation target
+        # take the namespace passed as installation target or None
         pass
     _func = f"{action}_{kind}"
     if api is None:
@@ -118,31 +141,32 @@ def k8s_call_api(
         kind_plural = {"ingress": "ingresses"}.get(
             obj["kind"].lower(), f"{obj['kind'].lower()}s"
         )
-        logger.debug(
-            f"Running a REST {action} operation for {obj['kind']} "
-            f"/apis/{obj['apiVersion']}/namespaces/{namespace}/{kind_plural}/"
-        )
-        if action == "create":
-            return config.K8S_CORE_API.api_client.call_api(
-                f"/apis/{obj['apiVersion']}/namespaces/{namespace}/{kind_plural}/",
-                "POST",
-                body=obj,
+        if namespace:
+            base_url = (
+                f"/apis/{obj['apiVersion']}/namespaces/{namespace}/{kind_plural}/"
             )
+            logger.debug(
+                f"Running a REST {action} operation for {obj['kind']} at {base_url}"
+            )
+        else:
+            base_url = f"/apis/{obj['apiVersion']}/{kind_plural}/"
+            logger.debug(
+                f"Running a REST {action} operation for {obj['kind']} at {base_url}"
+            )
+        if action == "create":
+            return config.K8S_CORE_API.api_client.call_api(base_url, "POST", body=obj)
         elif action == "patch":
             return config.K8S_CORE_API.api_client.call_api(
-                f"/apis/{obj['apiVersion']}/namespaces/{namespace}/{kind_plural}/{obj['metadata']['name']}",
-                "PUT",
-                body=obj,
+                f"{base_url}{obj['metadata']['name']}", "PUT", body=obj
             )
         elif action == "delete":
             from kubernetes.client.models.v1_delete_options import V1DeleteOptions
 
             delobj = V1DeleteOptions()
             return config.K8S_CORE_API.api_client.call_api(
-                f"/apis/{obj['apiVersion']}/namespaces/{namespace}/{kind_plural}/{obj['metadata']['name']}",
-                "DELETE",
-                body=delobj,
+                f"{base_url}{obj['metadata']['name']}", "DELETE", body=delobj
             )
+
         else:
             raise ValueError(f"The action {action} is not supported at the moment")
 
